/*
@BProCalcTableImport  @P@ICopyright Gold Disk Inc., January, 1993

This Genie will import a table of data from Professional Calc. You must have ProCalc running first and a box on the page for which the data is destined.
*/
numeric digits 8
cr  = '0a'x
options results

address command
call SafeEndEdit.rexx()
units = ppm_GetUnits()
if units = 3 then
    call ppm_SetUnits(1)
call ppm_AutoUpdate(0)

signal on halt
signal on break_c
signal on break_e
signal on break_d

if ~show(p, "PCALC") then exit_msg("Please start Professional Calc and select a range before running this Genie.")

box = ppm_ClickOnBox("Click on box in which to import table..")
if box = 0 then exit_msg()

/*  extract box attributes  */
boxsize = ppm_GetBoxSize(box)
boxposition = ppm_GetBoxPosition(box)

if ppm_Inform(2, "Delete box?",) = 1 then call ppm_DeleteBox(box)

/* tell user to start up ProCalc    */
call ppm_Inform(1, "Select Range in ProCalc",)
call ppm_ShowStatus("Extracting range from ProCalc")


/* switch to ProCalc and start processing   */
address "PCALC"
ActivateWindow
DrawMessage "Please enter Range"

'Suppress'
'LockGUI'

/*  get range selected by user  */
'Current'
range = upper(result)

if range     = '' then exit_msg("No range selected")
colon_pos    = pos(':', range)
if colon_pos = 0 then exit_msg("No range selected")

/*  extract the start cell and the end cell */
start_cell  = substr(range, 1, colon_pos - 1)
end_cell    = substr(range, colon_pos + 1)
if end_cell = '' | start_cell = '' then exit_msg("Invalid Range")

/*  make sure that we go from top left to bottom right  */
if start_cell > end_cell then
do
    temp        = start_cell
    start_cell  = end_cell
    end_cell    = temp
end

/*  change cell coordinates to numberic values  */
start_row   = cell_to_row( start_cell )
end_row     = cell_to_row( end_cell )
start_col   = cell_to_col( start_cell )
end_col     = cell_to_col( end_cell )


/*  calculate size of matrix    */
matrix.size = start_row * start_col

start_cell  = start_cell || ':'
save_start  = start_cell

'Position' start_cell

do row = start_row to end_row

    do col = start_col to end_col

        'IsValue'

        if result then
        do
            'GetValue'
            matrix.row.col  = result
        end
        else
        do
            'GetLabel'
            matrix.row.col  = result
        end

        'Cursor' 'Right'

    end

    'Position' start_cell
    'Cursor' 'Down'
    'Current'
    start_cell = result

end

'SelectRange' range

/*  calculate the dimensions of the new boxes   */
boxwidth    = word(boxsize, 1 ) / (end_col - start_col + 1 )
boxheight   = word(boxsize, 2 ) / (end_row - start_row + 1 )
boxleft     = word(boxposition, 1 )
boxtop      = word(boxposition, 2 )
boxright    = word(boxsize, 1) + boxleft
boxbottom   = word(boxsize, 2) + boxtop

'UnLockGUI'
'UnSuppress'

call ppm_AutoUpdate(0)

bleft   = boxleft
btop    = boxtop

selection   = "Right Justify Numbers"cr"Make Grid Lines"cr"Create Link"
selection   = upper(ppm_SelectFromList("Select Options", 30, 5, 1, selection))
if pos(GRID, selection) ~= 0 then grid = 1
else grid       = 0
if pos(LINK, selection) ~= 0 then link = 1
else link       = 0
if pos(RIGHT, selection) ~= 0 then right = 1
else right  = 0

prefix  = "PCALC"

if link then
do

    filename = ppm_GetFileName("Save Link Data", "", "PCalcLink.data")
    if filename = '' then exit_msg()

    if ~open(file, filename, "w") then call exit_msg("Unable to open file")

    call writeln(file, range)
    call writeln(file, save_start)
    call writeln(file, start_row)
    call writeln(file, end_row)
    call writeln(file, start_col)
    call writeln(file, end_col)
    call close(file)
    prefix  = fcomponent(filename)

end

call ppm_SetSize(12)

do row = start_row to end_row

    do col = start_col to end_col

        box = ppm_CreateBox(bleft, btop, boxwidth, boxheight, 0,prefix" "col"-"row)
        bleft   = bleft + boxwidth
        call ppm_SetBoxMargins(box, .1, .1, .1, .1)

        if grid then
        do
            call ppm_SetBoxFrame(box, 1)
            call ppm_SetBoxFrameData(box, "black", "black", .5, 1, 0)
        end

                if matrix.row.col = "" then matrix.row.col = " "

        if right & datatype(matrix.row.col) = 'NUM' then
            call ppm_TextIntoBox(box, "\jr"matrix.row.col)
        else
            call ppm_TextIntoBox(box, "\jl"matrix.row.col)
    end

    bleft   = boxleft
    btop    = btop  + boxheight

end


exit_msg("Done. You may use the ProCalcTableUpdate Genie to update the table if values change.")
break_d:
break_e:
break_c:
halt:
    call exit_msg("User aborted Genie!")

exit_msg: procedure expose units
do
    parse arg message

    call ppm_ClearStatus()
    if message ~= '' then call ppm_Inform(1, message,)
    call ppm_SetUnits(units)

        if show(p, "PCALC") then
        do
            address PCALC
            'UnSuppress'
            'UnLockGUI'
        end

        call ppm_AutoUpdate(1)
    exit
end

cell_to_row:
do
    arg cell

    row = ''

    do i = 1 to length(cell)
        char    = substr(cell,i, 1)
        if datatype(char) = 'NUM' then
            row = row || char
    end
    return row
end


cell_to_col:
do
    arg cell
    numbers = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    col     = 0

    do i = 1 to length(cell)
        char    = substr(cell,i, 1)
        if datatype(char) = 'NUM' then leave
        col     = col + ((26 ** (i - 1)) * pos(char, numbers))
    end
    return col
end

fcomponent: procedure
do
    arg filename

    slash = max(lastpos('/', filename), lastpos(':', filename))

    if slash = 0 then return(filename)
    else return(substr(filename, slash + 1))
end
